#!/bin/bash

# Version 04/05/2012

# Questo script ha il compito di intercettare la canzone attualmente
# in esecuzione (o comunque caricata) in xmms/mpg123/mplayer e creare
# opportunamente una stringa di comando IRC da spedire nel canale
# corrente tramite il comando "/budus pipe_script" di BuDuScRiPt
# plugin (Version 1.2.22 o superiore).

# http://digilander.libero.it/udasoft

#####################################################################
#____________________________________________________________________
# Uso: budus_mp3.sh <mpg/xmms/mpl/as/ak/tm/rb/ex/b/lfm> [[#chan] | [f] [auto] [noauto]]

#      budus_mp3.sh mpg
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da mpg123.

#      budus_mp3.sh xmms
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da xmms.

#      budus_mp3.sh mpl
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da mplayer.

#      budus_mp3.sh tm
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da timidity.

#      budus_mp3.sh rb
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da Rhythmbox.

#      budus_mp3.sh as
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da audacious.

#      budus_mp3.sh ak
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da amarok.

#      budus_mp3.sh ako
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da amarok, versione precedente alla 2.

#      budus_mp3.sh ex
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da exaile.

#      budus_mp3.sh b
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da banshee.

#      budus_mp3.sh lfm
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da Last.fm.

#      budus_mp3.sh mpg f
# Lo script tentera' di intercettare la canzone attualmente in
# riproduzione da mpg123; tentera' anche, nel medesimo messaggio, di
# comunicare anche l'indirizzo FTP (eventualmente presente sulla
# medesima macchina) che consente di scaricare la canzone in
# esecuzione.
# Affinche' la canzone sia scaricabile via FTP attraverso il link
# intercettato dallo script, e' necessario che il file mp3 sia in un
# path accessibile dal server ftp stesso e con opportuni permessi.

# Il parametro [auto] consente di automatizzare la scritta della
# canzone attualmente in ascolto in modo che venga aggiornata quando
# quest'ultima viene cambiata; il sistema si ferma nel momento in cui
# non viene piu' intercettata nessuna canzone in ascolto.
# Con il parametro [noauto] si ferma forzatamente l'esecuzione
# automatizzata, senza uscire dal player.

# Se, in modalita' automatica, si desidera inviare il messaggio in
# piu' di un canale e' necessario usare la sintassi:

#      budus_mp3.sh <mpg/xmms/mpl/as/ak/tm/rb/ex/b/lfm> <#chan>

# ad esempio

#      budus_mp3.sh as #miocanale

# inviera' il messaggio alla sessione #miocanale; in questo modo e'
# possibile aggiungere piu' canali e fare in modo che il messaggio
# venga visualizzato su tutti contemporaneamente.

# Non e' possibile togliere un canale una volta aggiunto, se non
# terminando la modalita' automatica con il parametro 'noauto' come
# descritto sopra (in questo modo si azzera la lista dei canali); cio'
# significa che la lista NON E' permanente, va ricreata ogni volta che
# si attiva la modalita' automatica.

# Creando il file budus_mp3.nolink anziche' il link al video, verranno
# emessi i dati caratteristici del file mp3.

# Creando il file budus_mp3_command.txt e' possibile personalizzare
# l'output dello script; all'interno del file e' possibile utilizzare
# le seguenti variabili per comporre la stringa da visualizzare nel
# canale:
# PLAYER_NAME		-> Player usato per riprodurre il media
# DEFAULT_DATA		-> Equivale a: [ MEDIA_TIME ][ MEDIA_BITRATE ][ MEDIA_SIZE ]
# MEDIA_FULLPATH	-> Percorso completo del media
# MEDIA_TIME		-> Durata del media
# MEDIA_BITRATE		-> Bitrate del media
# MEDIA_SIZE		-> Dimensione del media
# MEDIA_NAME		-> Nome del media (senza percorso ed estensione)
# MEDIA_FILENAME	-> Nome del file del media
# MEDIA_FILEPATH	-> Percorso del media, senza nome del file
# MEDIA_PARSED_NAME	-> Nome del media con la formattazione di default
# MEDIA_AUTHOR_NAME	-> Autore del media
# MEDIA_TITLE_NAME	-> Titolo del media
# MEDIA_VIDEO_URL	-> URL ad un video collegato al media
# MP3_FULLPATH		-> Eventuale percorso completo al file MP3
# FTP_CURRENT_IP	-> IP del Server FTP locale
# OUTPUT_TYPE		-> Tipoligia di output

# Le variabili di cui sopra non sono sempre disponibili; cio' dipende
# dal player audio usato.

# Se il media in ascolto e' un file mp3 (e non uno streaming) e
# risulta installato il software che fornisce il comando "id3info" e'
# possibile utilizzare questi ulteriori TAG (che vengono prelevati
# direttamente dal file):
# TAG_TITLE		-> Titolo
# TAG_ARTIST		-> Artista
# TAG_ALBUM		-> Album
# TAG_YEAR		-> Anno
# TAG_TRACK		-> Numero della Traccia
# TAG_COMMENT		-> Commento
# TAG_GENDER		-> Genere
# TAG_CODEC		-> Codec del file
# TAG_BITRATE		-> Bitrate
# TAG_FREQUENCY		-> Frequenza

# Affinche' "id3info" riesca a catturare le informazioni e' necessario
# che il player utilizzato fornisca il path completo al file; in caso
# di problemi e' possibile verificare se il path completo e' corretto
# facendo un test con la variabile "MEDIA_FULLPATH".


#####################################################################
#____________________________________________________________________
function user_command_parsing() {
    printf '%s\n' "${user_command}" \
	| sed "s@PLAYER_NAME@${PLAYER_NAME}@g" \
	| sed "s@MEDIA_FULLPATH@${MEDIA_FULLPATH}@g" \
	| sed "s@MEDIA_TIME@${MEDIA_TIME}@g" \
	| sed "s@MEDIA_BITRATE@${MEDIA_BITRATE}@g" \
	| sed "s@MEDIA_SIZE@${MEDIA_SIZE}@g" \
	| sed "s@DEFAULT_DATA@${DEFAULT_DATA}@g" \
	| sed "s@MEDIA_NAME@${MEDIA_NAME}@g" \
	| sed "s@MEDIA_FILENAME@${MEDIA_FILENAME}@g" \
	| sed "s@MEDIA_FILEPATH@${MEDIA_FILEPATH}@g" \
	| sed "s@MEDIA_PARSED_NAME@${MEDIA_PARSED_NAME}@g" \
	| sed "s@MEDIA_AUTHOR_NAME@${MEDIA_AUTHOR_NAME}@g" \
	| sed "s@MEDIA_TITLE_NAME@${MEDIA_TITLE_NAME}@g" \
	| sed "s@MP3_FULLPATH@${MP3_FULLPATH}@g" \
	| sed "s@FTP_CURRENT_IP@${FTP_CURRENT_IP}@g" \
	| sed "s@MEDIA_VIDEO_URL@${MEDIA_VIDEO_URL}@g" \
	| sed "s@OUTPUT_TYPE@${OUTPUT_TYPE}@g" \
	| sed "s@TAG_TITLE@${TAG_TITLE:-SCONOSCIUTO}@g" \
	| sed "s@TAG_ARTIST@${TAG_ARTIST:-SCONOSCIUTO}@g" \
	| sed "s@TAG_ALBUM@${TAG_ALBUM:-SCONOSCIUTO}@g" \
	| sed "s@TAG_YEAR@${TAG_YEAR:-SCONOSCIUTO}@g" \
	| sed "s@TAG_TRACK@${TAG_TRACK:-SCONOSCIUTA}@g" \
	| sed "s@TAG_COMMENT@${TAG_COMMENT:-SCONOSCIUTO}@g" \
	| sed "s@TAG_GENDER@${TAG_GENDER:-SCONOSCIUTO}@g" \
	| sed "s@TAG_CODEC@${TAG_CODEC:-SCONOSCIUTO}@g" \
	| sed "s@TAG_BITRATE@${TAG_BITRATE:-SCONOSCIUTO}@g" \
	| sed "s@TAG_FREQUENCY@${TAG_FREQUENCY:-SCONOSCIUTO}@g"
}

#####################################################################
#____________________________________________________________________
function make_command() {
    printf '[MP3 MANAGER] MP3_FULLPATH = %s\n' "${MP3_FULLPATH:-STREAMING}" >&2
    
    if [ -f "${budus_mp3_command_file}" ]; then
	case ${OUTPUT_TYPE-ERROR} in
	    "DEFAULT")
		if [ -f "${MP3_FULLPATH:-NULL.mp3}" ] && \
		    [ "$(echo "${MP3_FULLPATH:-NULL.mp3}" | grep -i '\.mp3$')" != "" ]; then
		    user_command="$(grep '^FILE:/' "${budus_mp3_command_file}" | cut -f2- -d':')"
		else
		    user_command="$(grep '^DEFAULT:/' "${budus_mp3_command_file}" | cut -f2- -d':')"
		fi
		;;
	    "FTP")
		user_command="$(grep '^FTP:/' "${budus_mp3_command_file}" | cut -f2- -d':')"
		;;
	    "VIDEO")
		user_command="$(grep '^VIDEO:/' "${budus_mp3_command_file}" | cut -f2- -d':')"
		;;
	    *)
		user_command=
		;;
	esac
	
	if [ "${user_command}" != "" ]; then
	    if [ -f "${MP3_FULLPATH:-NULL.mp3}" ]; then
		if [ $(which id3info 2> /dev/null) ]; then
		    mp3_tag_info=$(id3info "${MP3_FULLPATH:-NULL.mp3}")
		    
		    TAG_TITLE=$(echo "${mp3_tag_info}" | grep '^=== TIT2' | awk -F':' '{print $2}' | sed 's@^ *@@')
		    TAG_ARTIST=$(echo "${mp3_tag_info}" | grep '^=== TPE1' | awk -F':' '{print $2}' | sed 's@^ *@@')
		    TAG_ALBUM=$(echo "${mp3_tag_info}" | grep '^=== TALB' | awk -F':' '{print $2}' | sed 's@^ *@@')
		    TAG_YEAR=$(echo "${mp3_tag_info}" | grep '^=== TYER' | awk -F':' '{print $2}' | sed 's@^ *@@')
		    TAG_TRACK=$(echo "${mp3_tag_info}" | grep '^=== TRCK' | awk -F':' '{print $2}' | sed 's@^ *@@')
		    TAG_COMMENT=$(echo "${mp3_tag_info}" | grep '^=== COMM' | awk -F':' '{print $2}' | sed 's@^ *@@')
		    TAG_GENDER=$(echo "${mp3_tag_info}" | grep '^=== TCON' | awk -F':' '{print $2}' | sed 's@^ *@@')
		    
		    TAG_CODEC=$(echo "${mp3_tag_info}" | tail -n3 | head -n1)
		    TAG_BITRATE=$(echo "${mp3_tag_info}" | tail -n2 | head -n1 | awk -F':' '{print $2}' | sed 's@^ *@@')
		    TAG_FREQUENCY=$(echo "${mp3_tag_info}" | tail -n1 | awk -F':' '{print $2}' | sed 's@^ *@@')
		elif [ "$(echo "${user_command}" | grep -F -e 'TAG_TITLE' -e 'TAG_ARTIST' -e 'TAG_ALBUM' -e 'TAG_YEAR' -e 'TAG_TRACK' -e 'TAG_COMMENT' -e 'TAG_GENDER' -e 'TAG_CODEC' -e 'TAG_BITRATE' -e 'TAG_FREQUENCY')" != "" ]; then
		    printf '/ECHO [MP3 MANAGER]\t %%B%%C4,1ERRORE: %%O%%C4,1 Nel sistema non risulta installato il software che fornisce il comando %%Bid3info%%O; non sono in grado di estrapolare i TAG richiesti.\n'
		    printf '[MP3 MANAGER] Nel sistema non risulta installato il software che fornisce il comando  id3info ; non sono in grado di estrapolare i TAG richiesti.\n' >&2
		fi
	    fi
	    
	    mp3_command=$(user_command_parsing)
	else
	    mp3_command="/SAY ${mp3_command}"
	fi
    else
	mp3_command="/SAY ${mp3_command}"
    fi
}

#####################################################################
#____________________________________________________________________
function print_song() {
    make_command
    
    if [ -f "${channel_list_file}" ]; then
	for channel in $(cat "${channel_list_file}" | sort | uniq); do
	    echo "/budus sendsess ${channel} /budus if \$nickname.away = 0 ${mp3_command}"
	done
    else
	echo "/budus if \$nickname.away = 0 ${mp3_command}"
    fi
}

#####################################################################
#____________________________________________________________________
script_path=$(echo $0 | sed 's@/budus_mp3.sh@@')

xmms_infofile=/tmp/xmms-info
msg_refresh=15
autospam_file="${script_path}/mp3_autospam.dat"
autospam_file_stop="${script_path}/mp3_autospam.lock"
channel_list_file="${script_path}/mp3_chanlist.dat"
budus_mp3_command_file="${script_path}/budus_mp3_command.txt"

#####################################################################
#____________________________________________________________________
if [ ${2:-X} == "MAKE_USER_COMMAND_FILE" ]; then
    if [ ! -f "${budus_mp3_command_file}" ]; then
	printf '/ECHO Il file %s non esiste; ne creo uno in automatico.\n' "${budus_mp3_command_file}"
	
	cat<<EOF > "${budus_mp3_command_file}"
# BuDuScRiPt plugin - MP3 Manager User Command File
# $(date)
# Le seguenti 4 righe servono per personalizzare gli annunci della
# canzone in ascolto:

DEFAULT:/ACTION ascolta con PLAYER_NAME: MEDIA_PARSED_NAME DEFAULT_DATA
FILE:/ACTION ascolta con PLAYER_NAME: TAG_ARTIST - TAG_TITLE (TAG_ALBUM - TAG_YEAR) [TAG_BITRATE:TAG_FREQUENCY]
FTP:/ACTION ascolta con PLAYER_NAME: MEDIA_PARSED_NAME (OUTPUT_TYPE: FTP_CURRENT_IP/MEDIA_FULLPATH)
VIDEO:/ACTION ascolta con PLAYER_NAME: MEDIA_PARSED_NAME (OUTPUT_TYPE: MEDIA_VIDEO_URL)

# All'interno del file e' possibile utilizzare le seguenti variabili per
# comporre la stringa da visualizzare nel canale:
#
# PLAYER_NAME		-> Player usato per riprodurre il media
# DEFAULT_DATA		-> Equivale a: [ MEDIA_TIME ][ MEDIA_BITRATE ][ MEDIA_SIZE ]
# MEDIA_FULLPATH	-> Percorso completo del media
# MEDIA_TIME		-> Durata del media
# MEDIA_BITRATE		-> Bitrate del media
# MEDIA_SIZE		-> Dimensione del media
# MEDIA_NAME		-> Nome del media (senza percorso ed estensione)
# MEDIA_FILENAME	-> Nome del file del media
# MEDIA_FILEPATH	-> Percorso del media, senza nome del file
# MEDIA_PARSED_NAME	-> Nome del media con la formattazione di default
# MEDIA_AUTHOR_NAME	-> Autore del media
# MEDIA_TITLE_NAME	-> Titolo del media
# MEDIA_VIDEO_URL	-> URL ad un video collegato al media
# MP3_FULLPATH		-> Eventuale percorso completo al file MP3
# FTP_CURRENT_IP	-> IP del Server FTP locale
# OUTPUT_TYPE		-> Tipoligia di output

# Le variabili di cui sopra non sono sempre disponibili; cio' dipende
# dal player audio usato.

# Se il media in ascolto e' un file mp3 (e non uno streaming) e
# risulta installato il software che fornisce il comando "id3info" e'
# possibile utilizzare questi ulteriori TAG (che vengono prelevati
# direttamente dal file):
# TAG_TITLE		-> Titolo
# TAG_ARTIST		-> Artista
# TAG_ALBUM		-> Album
# TAG_YEAR		-> Anno
# TAG_TRACK		-> Numero della Traccia
# TAG_COMMENT		-> Commento
# TAG_GENDER		-> Genere
# TAG_CODEC		-> Codec del file
# TAG_BITRATE		-> Bitrate
# TAG_FREQUENCY		-> Frequenza

# Affinche' "id3info" riesca a catturare le informazioni e' necessario
# che il player utilizzato fornisca il path completo al file; in caso
# di problemi e' possibile verificare se il path completo e' corretto
# facendo un test con la variabile "MEDIA_FULLPATH".
EOF
    fi
    
    [ -f "${budus_mp3_command_file}" ] && \
	[ $(which nano 2> /dev/null) ] && \
	printf '/ECHO Ho individuato un editor (NANO); i suoi comandi sono usabili premendo il tasto CTRL .\n' && \
	echo "/EXEC xterm -e nano \"${budus_mp3_command_file}\""
    
    exit 0;
fi

#####################################################################
#____________________________________________________________________
if [ $# -lt 1 ]; then
    echo "Parametri insufficienti. Specificare quale player sta riproducendo l'mp3."
    exit 1;
fi

#####################################################################
#____________________________________________________________________
get_channel=$(echo "$*" | grep -F "#" | sed s/.*#/#/ | cut -f1 -d' ')
if [ "${get_channel}" != "" ]; then
    echo "${get_channel}" >> "${channel_list_file}"
    echo "/ECHO %C7,1Channel %B${get_channel}%O%C7,1 added."
    
    exit 1;
fi

#####################################################################
#____________________________________________________________________
play_command=$1
if [ "$2" == "f" ] || [ "$3" == "f" ]; then
    ftp_link=yes
else
    ftp_link=no
fi

#####################################################################
#____________________________________________________________________
skip_autostop_flag=0
if ([ "$2" == "auto" ] || [ "$3" == "auto" ]) || [ -f "${autospam_file}" ]; then
    auto_spam=yes
    if [ ! -f "${autospam_file}" ]; then
	echo "/ECHO %C3,1Auto-Spam Mp3 Activated"
	touch "${autospam_file}"
	skip_autostop_flag=1
    fi
else
    auto_spam=no
fi

#####################################################################
#____________________________________________________________________
if ([ "$2" == "noauto" ] || [ "$3" == "noauto" ]); then
    echo "/ECHO %C5,1Auto-Spam Mp3 Deactivating in progress ..."
    touch "${autospam_file_stop}"
    
    exit 1;
elif ([ "$2" == "auto" ] || [ "$3" == "auto" ]) && [ -f ${autospam_file_stop} ] && [ ${skip_autostop_flag} == 0 ]; then
    if [ -f "${autospam_file}" ]; then
	echo "/ECHO %C4,1Auto-Spam Mp3 Deactivated"
	rm -f "${autospam_file}"
	rm -f "${channel_list_file}"
    fi
    
    rm -f "${autospam_file_stop}"
    
    exit 1;
fi

#####################################################################
#____________________________________________________________________
spam_mex="BuDuScRiPt rul3z"

if [ "${play_command}" == "mpg" ]; then
    PLAYER_NAME="mpg123"
    MP3_FILES_OPENED=$([ $(which lsof 2> /dev/null) ] && \
	lsof -c mpg123 \
	| grep -i '\.mp3$' \
	| sed 's@.*[0-9]\{2,\}[ ]*[0-9]\{2,\} /@/@' || \
	echo "[MP3 MANAGER] Software  lsof  non trovato." >&2)
    
    MEDIA_FULLPATH=$(echo "${MP3_FILES_OPENED}" \
	| tail -n1)
    
    MP3_FULLPATH="${MEDIA_FULLPATH}"
elif [ "${play_command}" == "xmms" ]; then
    PLAYER_NAME="XMMS"
    if [ -e "${xmms_infofile}" ]; then
	MP3_FILES_OPENED="$(cat ${xmms_infofile})"
	MEDIA_FULLPATH="$(echo "${MP3_FILES_OPENED}" | grep -F "File:" | sed 's/File: //')"
	MP3_FULLPATH="${MEDIA_FULLPATH}"
    else
	if [ -f "${autospam_file}" ]; then
	    echo "/ECHO %C4,1Auto-Spam Mp3 Deactivated (NO XMMS INFO FILE)"
	    rm -f "${autospam_file}"
	    rm -f "${channel_list_file}"
	else
	    echo "/ECHO Impossibile determinare il nome del file in riproduzione."
	    echo "/ECHO Verificare che il plugin InfoPipe per XMMS sia Abilitato."
	fi
	
	exit 1;
    fi
elif [ "${play_command}" == "mpl" ]; then
    PLAYER_NAME="MPlayer"
    MP3_FILES_OPENED=$([ $(which lsof 2> /dev/null) ] && \
	lsof -c mplayer \
	| grep -i '\.mp3$' \
	| sed 's@.*[0-9]\{2,\}[ ]*[0-9]\{2,\} /@/@' || \
	echo "[MP3 MANAGER] Software  lsof  non trovato." >&2)
    
    MEDIA_FULLPATH=$(echo "${MP3_FILES_OPENED}" \
	| tail -n1)
    
    MP3_FULLPATH="${MEDIA_FULLPATH}"
elif [ "${play_command}" == "tm" ]; then
    PLAYER_NAME="Timidity"
    MP3_FILES_OPENED="$(ps -ef \
	| grep -iF " timidity " \
	| grep -i '\.mid$')"
    
    MEDIA_FULLPATH=$(echo "${MP3_FILES_OPENED}" \
	| head -n1 \
	| sed 's/.*timidity //g' \
	| sed 's/-[0-9a-zA-Z]* //')
    
    MP3_FULLPATH="${MEDIA_FULLPATH}"
elif [ "${play_command}" == "rb" ]; then
    PLAYER_NAME="Rhythmbox"
    if [ "$(rhythmbox-client --help 2> /dev/null)" == "" ]; then
	echo "/ECHO Non risulta installato rhythmbox-client per Rhythmbox. Impossibile intercettare la canzone in ascolto."
	
	exit 1;
    fi
    MEDIA_FULLPATH="$(rhythmbox-client --print-playing)"
    if [ "$(echo "${MEDIA_FULLPATH}" | grep -iF 'http://')" ]; then
	flag_media_content_stream=1
	MEDIA_TIME="STREAMING"
	MEDIA_BITRATE="STREAMING"
	MEDIA_SIZE="STREAMING"
    else
	MP3_FILES_OPENED=$([ $(which lsof 2> /dev/null) ] && \
	    lsof -c rhythmbox \
	    | grep -i '\.mp3$' \
	    | sed 's@.*[0-9]\{2,\}[ ]*[0-9]\{2,\} /@/@' || \
	    echo "[MP3 MANAGER] Software  lsof  non trovato." >&2)
	
	MP3_FULLPATH=$(echo "${MP3_FILES_OPENED}" \
	    | grep -iF "${MEDIA_FULLPATH}" \
	    | tail -n1)

	if [ "${MP3_FULLPATH}" ]; then
	    MP3_FULLPATH=$(echo "${MP3_FILES_OPENED}" \
		| tail -n1)
	fi
	
	if [ "${MP3_FILES_OPENED}" == "" ]; then
	    echo "/ECHO Impossibile individuare la canzone in ascolto. Verificare che Rhythmbox stia effettivamente riproducendo un file MP3."
	    
	    exit 1;
	else
	    if [ "${MP3_FULLPATH}" == "" ]; then
		MP3_FULLPATH="${MP3_FILES_OPENED}"
	    fi
	fi
	
	MEDIA_TIME="$(rhythmbox-client --print-playing-format %td | tr '.' ':' | sed 's@[^0-9:]*@@')"
	[ "${MEDIA_TIME}" == "" ] && MEDIA_TIME="0:00:00"
	MEDIA_BITRATE="?KHz"
	[ -f "${MP3_FULLPATH}" ] && MEDIA_SIZE="$(du -h "${MP3_FULLPATH}" | cut -f1)"
	DEFAULT_DATA=$(printf "[ %s ][ %s ][ %s ]" \
	    "${MEDIA_TIME}" \
	    "${MEDIA_BITRATE}" \
	    "${MEDIA_SIZE}")
    fi
elif [ "${play_command}" == "as" ]; then
    PLAYER_NAME="Audacious"
    if [ "$(audtool2 help 2> /dev/null)" != "" ]; then
	audtool=audtool2
    else
	audtool=audtool
	if [ "$(audtool2 help 2> /dev/null)" == "" ]; then
	    echo "/ECHO Non risulta installato AudTool per Audacious. Impossibile intercettare la canzone in ascolto."
	    
	    exit 1;
	fi
    fi
    
    MP3_FILES_OPENED="$(${audtool} --current-song-filename)"
    MEDIA_FULLPATH="$(echo "${MP3_FILES_OPENED}" | sed 's/%20/ /g')"
    if [ "$(echo "${MEDIA_FULLPATH}" | grep -iF 'http://')" ]; then
	flag_media_content_stream=1
	MEDIA_FULLPATH="$(${audtool} --current-song)"
    else
	temp="$(${audtool} --current-song-filename)"
	if [ $? != 0 ] || [ "$(${audtool} --current-song-filename | grep "^http://")" != "" ]; then
	    MEDIA_FULLPATH="$(${audtool} --current-song | sed 's/%20/ /g')"
	    temp="$(${audtool} --current-song)"
	    if [ $? != 0 ]; then
		MEDIA_FULLPATH=""
	    fi
	fi
    fi
    
    MEDIA_TIME="$(${audtool} current-song-length)"
    MEDIA_BITRATE="$(${audtool} current-song-frequency-khz)KHz"
    MP3_FILES_OPENED="$(${audtool} current-song-filename)"
    MP3_FULLPATH="$(echo "${MP3_FILES_OPENED}" | sed s@file://@@ | sed s/%20/\ /g)"
    [ -f "${MP3_FULLPATH}" ] && MEDIA_SIZE="$(du -h "${MP3_FULLPATH}" | cut -f1)"
    DEFAULT_DATA=$(printf "[ %s ][ %s ][ %s ]" \
	"${MEDIA_TIME:-0}" \
	"${MEDIA_BITRATE:-0}" \
	"${MEDIA_SIZE:-STREAMING}")
elif [ "${play_command}" == "ak" ]; then
    PLAYER_NAME="Amarok"
    if [ "$(qdbus --help 2> /dev/null)" == "" ]; then
	echo "/ECHO Non risulta installato qdbus (qt-dbus). Impossibile collegarsi con Amarok per intercettare la canzone in ascolto."
	
	exit 1;
    fi
    
    MP3_FILES_OPENED="$(qdbus org.kde.amarok /Player GetMetadata)"
    MEDIA_FULLPATH="$(echo "${MP3_FILES_OPENED}" | grep -i '^location: ' | sed 's/^location: //' | sed 's/%20/ /g')"
    if [ "$(echo "${MEDIA_FULLPATH}" | grep -iF 'http://')" ]; then
	flag_media_content_stream=1
    fi
    
    MEDIA_TIME="$(qdbus org.kde.amarok /Player GetMetadata | grep -i '^time: ' | sed 's/^time: //')"

    if [ "$(echo "${MEDIA_TIME:-0}" | grep '^[0-9]*:[0-9]*\(:[0-9]*\)\?$')" != "" ]; then
	time_H=$(echo "${MEDIA_TIME:-0}" | grep -o '^[0-9]*:' | cut -f1 -d':')
	time_M=$(echo "${MEDIA_TIME:-0}" | grep -o ':[0-9]*:\?' | sed 's@:@@g')
	time_S=$(echo "${MEDIA_TIME:-0}" | grep -o ':[0-9]*:[0-9]*$' | sed 's@:@@' | cut -f2- -d':')
	
	time_H="${time_H:-0}"
	time_M="${time_M:-00}"
	time_S="${time_S:-00}"
    else
	if [ ${MEDIA_TIME:-0} -ge 3600 ]; then
	    time_H=$((${MEDIA_TIME} / 3600))
	    time_Mt=$((${MEDIA_TIME} - $((3600 * ${time_H}))))
	else
	    time_H=0
	    time_Mt=${MEDIA_TIME:-0}
	fi
	if [ ${time_Mt} -ge 60 ]; then
	    time_M=$((${time_Mt} / 60))
	    time_S=$((${time_Mt} - $((60 * ${time_M}))))
	else
	    time_M=0
	    time_S=${time_Mt}
	fi
	
	if [ ${time_M} -lt 10 ]; then
	    time_M="0${time_M}"
	fi
	if [ ${time_S} -lt 10 ]; then
	    time_S="0${time_S}"
	fi
    fi
    
    MEDIA_TIME="${time_H}:${time_M}:${time_S}"
    MEDIA_BITRATE="$(echo "${MP3_FILES_OPENED}" | grep -i '^audio-bitrate: ' | sed 's/^audio-bitrate: //')"
    MP3_FULLPATH="$(echo "${MEDIA_FULLPATH}" | sed s@file://@@ | sed 's/%20/ /g')"
    [ -f "${MP3_FULLPATH}" ] && MEDIA_SIZE="$(du -h "${MP3_FULLPATH}" | cut -f1)"
    DEFAULT_DATA=$(printf "[ %s ][ %s ][ %s ]" \
	"${MEDIA_TIME}" \
	"${MEDIA_BITRATE} bit" \
	"${MEDIA_SIZE}")
elif [ "${play_command}" == "ako" ]; then
    PLAYER_NAME="Amarok"
    if [ "$(dcop 2> /dev/null)" == "" ]; then
	echo "/ECHO Non risulta installato DCOP . Impossibile collegarsi con Amarok per intercettare la canzone in ascolto."
	
	exit 1;
    fi
    
    MP3_FILES_OPENED="$(dcop amarok player path)"
    MEDIA_FULLPATH="$(echo "${MP3_FILES_OPENED}" | sed s/%20/\ /g)"
    if [ "$(echo "${MEDIA_FULLPATH}" | grep -iF 'http://')" ]; then
	flag_media_content_stream=1
    fi
    
    temp="${MP3_FILES_OPENED}"
    if [ $? != 0 ] || [ "$(dcop amarok player path | grep "^http://")" != "" ]; then
	MEDIA_FULLPATH="$(dcop amarok player title | sed s/%20/\ /g)"
	temp="$(dcop amarok player title)"
	if [ $? != 0 ]; then
	    MEDIA_FULLPATH=""
	fi
    fi
    
    MEDIA_TIME="$(dcop amarok player totalTime)"
    MEDIA_BITRATE="$(dcop amarok player bitrate) bit"
    MP3_FULLPATH="$(dcop amarok player path | sed s@file://@@ | sed 's/%20/ /g')"
    [ -f "${MP3_FULLPATH}" ] && MEDIA_SIZE="$(du -h "${MP3_FULLPATH}" | cut -f1)"
    DEFAULT_DATA=$(printf "[ %s ][ %s ][ %s ]" \
	"${MEDIA_TIME}" \
	"${MEDIA_BITRATE}" \
	"${MEDIA_SIZE}")
elif [ "${play_command}" == "ex" ]; then
    PLAYER_NAME="Exaile"
    if [ "$(qdbus --help 2> /dev/null)" == "" ]; then
	echo "/ECHO Non risulta installato qdbus (qt-dbus). Impossibile collegarsi con Exaile per intercettare la canzone in ascolto."
	
	exit 1;
    fi
    
    find_exaile_dbbus_service=$(qdbus | grep -iF 'exaile' | awk '{print $1}')
    if [ "${find_exaile_dbbus_service}" == "" ]; then
	echo "/ECHO Non risulta possibile collegarsi con Exaile per intercettare la canzone in ascolto (MISSING EXAILE DBUS)."
	
	exit 1;
    fi

    find_exaile_dbbus_path=$(qdbus "${find_exaile_dbbus_service}" | tail -n 1 | awk '{print $1}')
    GetTrackAttr_params__artist="artist"
    GetTrackAttr_params__title="title"
    
    case ${find_exaile_dbbus_path} in
	/org/exaile/Exaile)
	    GetTrackAttr="GetTrackAttr"
	    GetTrackAttr_params__loc="__loc"
	    GetTrackAttr_params__length="__length"
	    ;;
	/DBusInterfaceObject)
	    GetTrackAttr="get_track_attr"
	    GetTrackAttr_params__loc="loc"
	    GetTrackAttr_params__length="length"
	    ;;
	*)
	    echo "/ECHO Non risulta possibile collegarsi con Exaile per intercettare la canzone in ascolto (EXAILE DBUS UNSUPPORTED)."
	    
	    exit 1;
	    ;;
    esac
    
    MP3_FILES_OPENED="$(qdbus ${find_exaile_dbbus_service} ${find_exaile_dbbus_path} ${GetTrackAttr} ${GetTrackAttr_params__loc})"
    MEDIA_FULLPATH="$(echo "${MP3_FILES_OPENED}" | sed 's@file://@@' | sed 's/%20/ /g')"
    if [ "$(echo "${MEDIA_FULLPATH}" | grep -iF 'http://')" ]; then
	flag_media_content_stream=1
    fi
    MEDIA_TIME="$(qdbus ${find_exaile_dbbus_service} ${find_exaile_dbbus_path} ${GetTrackAttr} ${GetTrackAttr_params__length} | cut -f1 -d'.')"
    
    if [ "${MEDIA_FULLPATH/http:*/LINK}" == "LINK" ]; then
	flag_media_content_stream=1
	MEDIA_FULLPATH="$(qdbus ${find_exaile_dbbus_service} ${find_exaile_dbbus_path} ${GetTrackAttr} ${GetTrackAttr_params__artist}) - $(qdbus ${find_exaile_dbbus_service} ${find_exaile_dbbus_path} ${GetTrackAttr} ${GetTrackAttr_params__title})"
	
	if [ "${MEDIA_TIME}" == "None" ]; then
	    MEDIA_TIME="0"
	fi
    fi
    
    if [ "$(echo "${MEDIA_TIME:-0}" | grep '^[0-9]*:[0-9]*\(:[0-9]*\)\?$')" != "" ]; then
	time_H=$(echo "${MEDIA_TIME:-0}" | grep -o '^[0-9]*:' | cut -f1 -d':')
	time_M=$(echo "${MEDIA_TIME:-0}" | grep -o ':[0-9]*:\?' | sed 's@:@@g')
	time_S=$(echo "${MEDIA_TIME:-0}" | grep -o ':[0-9]*:[0-9]*$' | sed 's@:@@' | cut -f2- -d':')
	
	time_H="${time_H:-0}"
	time_M="${time_M:-00}"
	time_S="${time_S:-00}"
    else
	if [ ${MEDIA_TIME:-0} -ge 3600 ]; then
	    time_H=$((${MEDIA_TIME} / 3600))
	    time_Mt=$((${MEDIA_TIME} - $((3600 * ${time_H}))))
	else
	    time_H=0
	    time_Mt=${MEDIA_TIME:-0}
	fi
	if [ ${time_Mt} -ge 60 ]; then
	    time_M=$((${time_Mt} / 60))
	    time_S=$((${time_Mt} - $((60 * ${time_M}))))
	else
	    time_M=0
	    time_S=${time_Mt}
	fi
	
	if [ ${time_M} -lt 10 ]; then
	    time_M="0${time_M}"
	fi
	if [ ${time_S} -lt 10 ]; then
	    time_S="0${time_S}"
	fi
    fi
    
    MEDIA_TIME="${time_H}:${time_M}:${time_S}"
    MEDIA_BITRATE="UNKNOWN"
    MP3_FULLPATH="${MEDIA_FULLPATH}"
    [ ${flag_media_content_stream:-0} == 0 ] && \
	[ -f "${MP3_FULLPATH}" ] && \
	MEDIA_SIZE="$(du -h "${MEDIA_FULLPATH}" | awk '{print $1}')" || \
	MEDIA_SIZE="STREAMING"
    DEFAULT_DATA=$(printf "[ %s ][ %s ][ %s ]" \
	"${MEDIA_TIME}" \
	"${MEDIA_BITRATE} bit" \
	"${MEDIA_SIZE}")
elif [ "${play_command}" == "b" ]; then
    PLAYER_NAME="Banshee"
    if [ "$(banshee --version 2> /dev/null)" == "" ]; then
	echo "/ECHO Non risulta installato banshee. Impossibile collegarsi con Banshee per intercettare la canzone in ascolto."
	
	exit 1;
    fi
    
    MP3_FILES_OPENED="$(banshee --query-uri)"
    MEDIA_FULLPATH="$(echo "${MP3_FILES_OPENED}" | sed 's@uri: @@'  | sed 's@file://@@' | sed 's/%20/ /g')"
    if [ "$(echo "${MEDIA_FULLPATH}" | grep -iF 'http://')" ]; then
	flag_media_content_stream=1
	MEDIA_FULLPATH="$(banshee --query-artist | sed 's@^artist: @@') - $(banshee --query-title | sed 's@^title: @@')"
	MEDIA_TIME="STREAMING"
	MEDIA_BITRATE="STREAMING"
	MEDIA_SIZE="STREAMING"
    else    
	MEDIA_TIME="$(banshee --query-duration | sed 's@duration: @@' | cut -f1 -d',')"
	
	if [ "$(echo "${MEDIA_TIME:-0}" | grep '^[0-9]*:[0-9]*\(:[0-9]*\)\?$')" != "" ]; then
	    time_H=$(echo "${MEDIA_TIME:-0}" | grep -o '^[0-9]*:' | cut -f1 -d':')
	    time_M=$(echo "${MEDIA_TIME:-0}" | grep -o ':[0-9]*:\?' | sed 's@:@@g')
	    time_S=$(echo "${MEDIA_TIME:-0}" | grep -o ':[0-9]*:[0-9]*$' | sed 's@:@@' | cut -f2- -d':')
	    
	    time_H="${time_H:-0}"
	    time_M="${time_M:-00}"
	    time_S="${time_S:-00}"
	else
	    if [ ${MEDIA_TIME:-0} -ge 3600 ]; then
		time_H=$((${MEDIA_TIME} / 3600))
		time_Mt=$((${MEDIA_TIME} - $((3600 * ${time_H}))))
	    else
		time_H=0
		time_Mt=${MEDIA_TIME:-0}
	    fi
	    if [ ${time_Mt} -ge 60 ]; then
		time_M=$((${time_Mt} / 60))
		time_S=$((${time_Mt} - $((60 * ${time_M}))))
	    else
		time_M=0
		time_S=${time_Mt}
	    fi
	    
	    if [ ${time_M} -lt 10 ]; then
		time_M="0${time_M}"
	    fi
	    if [ ${time_S} -lt 10 ]; then
		time_S="0${time_S}"
	    fi
	fi
	
	MEDIA_TIME="${time_H}:${time_M}:${time_S}"
	MEDIA_BITRATE="$(banshee --query-bit-rate | sed 's@bit-rate: @@')K"
	MP3_FULLPATH="${MEDIA_FULLPATH}"
	[ -f "${MP3_FULLPATH}" ] && MEDIA_SIZE="$(du -h "${MP3_FULLPATH}" | awk '{print $1}')"
    fi
    
    DEFAULT_DATA=$(printf "[ %s ][ %s ][ %s ]" \
	"${MEDIA_TIME}" \
	"${MEDIA_BITRATE} bit" \
	"${MEDIA_SIZE}")
elif [ "${play_command}" == "lfm" ]; then
    last_fm_log_file_path="${HOME}/.local/share/Last.fm/Last.fm.log"
    
    if [ -f ${last_fm_log_file_path} ]; then
	last_song_played_record=$(cat ${last_fm_log_file_path} \
	    | grep -iF 'Event::TrackMetaDataAvailable' \
	    | tail -n1 \
	    | sed 's@[ ]*Event::TrackMetaDataAvailable @@' \
	    | uniq \
	    | sed 's@ ? @\t@' \
	    | sed 's@ [0-9]*:[0-9]*$@\t&@')
	
	PLAYER_NAME="Last.fm"
	MP3_FILES_OPENED="[DB]"
	MEDIA_FULLPATH=""
	MP3_FULLPATH=""
	
	MEDIA_TIME="$(echo "${last_song_played_record}" | awk -F'\t' '{print $3}')"
	
	MEDIA_BITRATE="Last.fm"
	MEDIA_SIZE="Last.fm"
	DEFAULT_DATA=$(printf "[ %s ][ %s ][ %s ]" \
	    "${MEDIA_TIME}" \
	    "${MEDIA_BITRATE} bit" \
	    "${MEDIA_SIZE}")
	
	MEDIA_NAME="$(echo "${last_song_played_record}" | awk -F'\t' '{print $1" - "$2}')"
	MEDIA_FILENAME="$(echo "${last_song_played_record}" | awk -F'\t' '{print $1" - "$2}')"
	MEDIA_FILEPATH="http://last.fm"
	MEDIA_AUTHOR_NAME="$(echo "${last_song_played_record}" | awk -F'\t' '{print $1}')"
	MEDIA_TITLE_NAME="$(echo "${last_song_played_record}" | awk -F'\t' '{print $2}')"
	MEDIA_PARSED_NAME="%B${MEDIA_AUTHOR_NAME}%O - %B${MEDIA_TITLE_NAME}%O"
	
	MEDIA_FULLPATH="${MEDIA_AUTHOR_NAME} - ${MEDIA_TITLE_NAME}"
	
	OUTPUT_TYPE="DEFAULT"
	
	flag_media_content_stream=1
    else
	echo "/ECHO Non risulta installato Last.fm player. Impossibile collegarsi con Last.fm per intercettare la canzone in ascolto."
	
	exit 1;
    fi
else
    echo "Parametri errati. Specificare quale player sta riproducendo l'mp3 (mpg/xmms/mpl/as/ak/tm/rb/ex/b/lfm)."
    exit 1;
fi

#####################################################################
#____________________________________________________________________
if [ ${flag_media_content_stream:-0} == 0 ]; then
    MEDIA_NAME=$(basename "${MP3_FULLPATH}" | sed 's@\.[^.]*$@@')
    MEDIA_FILENAME=$(basename "${MP3_FULLPATH}")
    MEDIA_FILEPATH=$(dirname "${MP3_FULLPATH}")
    
    if [ "${MEDIA_NAME}" == "" ]; then
	if [ -f "${autospam_file}" ]; then
	    echo "/ECHO %C4,1Auto-Spam Mp3 Deactivated (NO FILE FOUND)"
	    rm -f "${autospam_file}"
	    rm -f "${channel_list_file}"
	    
	    exit 1;
	else
	    cat<<EOF
/ECHO Non risulta in esecuzione nessun brano. E' possibile siano presenti problemi nell'individuazione; variabili principali:
/ECHO MP3_FILES_OPENED = ${MP3_FILES_OPENED:-EMPTY}
/ECHO MP3_FULLPATH     = ${MP3_FULLPATH:-EMPTY}
/ECHO MEDIA_NAME       = ${MEDIA_NAME:-EMPTY}
EOF
	    MEDIA_NAME="Anacleto ReZZoniko - I'm a LamaH"
	fi
    fi
    
    MEDIA_PARSED_NAME=$(echo "%B ${MEDIA_NAME}" | sed 's/ - /%O -%B /' | sed 's@$@%O@')
    MEDIA_AUTHOR_NAME=$(echo "${MEDIA_NAME}" | sed 's@ - .*@@')
    MEDIA_TITLE_NAME=$(echo "${MEDIA_NAME}" | sed 's@.* - @@')
    
    OUTPUT_TYPE="DEFAULT"
else
    MEDIA_NAME=$(echo "${MEDIA_FULLPATH}" | sed 's@(.*)@@g' | sed 's@.*[>]@@')
    MEDIA_FILENAME="${MEDIA_FULLPATH}"
    MEDIA_FILEPATH=$(echo "${MEDIA_FULLPATH}" | grep -o 'http://[a-zA-Z0-9+._%?#/=&:-]*')
    MEDIA_PARSED_NAME=$(echo "%B ${MEDIA_NAME}" | sed 's/ - /%O -%B /' | sed 's@$@%O@')
    MEDIA_AUTHOR_NAME=$(echo "${MEDIA_NAME}" | sed 's@ - .*@@')
    MEDIA_TITLE_NAME=$(echo "${MEDIA_NAME}" | sed 's@.* - @@')
    
    OUTPUT_TYPE="DEFAULT"
fi

#####################################################################
#____________________________________________________________________
is_ftp_service_active=$(ps -e | grep ftpd | grep -v grep)
if [ "${is_ftp_service_active}" != "" ] && \
    [ "${ftp_link}" == "yes" ] && \
    [ ${flag_media_content_stream:-0} == 0 ]; then
    FTP_CURRENT_IP=$(/sbin/ifconfig | grep P-t-P | cut -f2- -d: | cut -f1 -d' ')
    
    if [ "${FTP_CURRENT_IP}" == "" ]; then
	FTP_CURRENT_IP=$(lynx -dump http://www.ip-adress.com \
	    | grep -F 'My IP address is: ' \
	    | sed 's/My IP address is: //g' \
	    | grep "^\([0-9]\|[0-9][0-9]\|[0-9][0-9][0-9]\)\.\([0-9]\|[0-9][0-9]\|[0-9][0-9][0-9]\)\.\([0-9]\|[0-9][0-9]\|[0-9][0-9][0-9]\)\.\([0-9]\|[0-9][0-9]\|[0-9][0-9][0-9]\)$")
    fi
    
    MEDIA_FULLPATH=$(echo "${MEDIA_FULLPATH}"\
	| sed s/\ /%20/g\
	| sed s/\(/%28/g\
	| sed s/\)/%29/g\
	| sed s/\'/%27/g\
	| sed 's@/home/ftp/@@'\
	| sed 's@file:///@@')
    mp3_command="%Bmp3%Oz ::${MEDIA_PARSED_NAME}%O :: ( ftp://${FTP_CURRENT_IP}/${MEDIA_FULLPATH} )"
    OUTPUT_TYPE="FTP"
else
    if [ ! -f ${script_path}/budus_mp3.nolink ]; then
	MEDIA_VIDEO_URL=$(${script_path}/budus_google_search.sh '!search +Video '"${MEDIA_NAME}" \
	    | grep '^/SAY ' \
	    | grep -v '^/SAY Spiacente' \
	    | grep -v '\.html$' \
	    | head -n1 \
	    | sed 's@/SAY %B@@')
    fi
    if [ ${MEDIA_VIDEO_URL:-NONE} == NONE ]; then
	echo "[MP3 MANAGER] NO DATA FOUND" >&2
	mp3_command="${PLAYER_NAME} ::${MEDIA_PARSED_NAME}%O :: ( ${DEFAULT_DATA} )"
    else
	mp3_command="${PLAYER_NAME} ::${MEDIA_PARSED_NAME}%O :: ( ${MEDIA_VIDEO_URL} )"
	OUTPUT_TYPE="VIDEO"
    fi
fi

#####################################################################
#____________________________________________________________________
if [ "${auto_spam}" == "yes" ]; then
    previous_song=$(head -n1 "${autospam_file}")

    if [ "${previous_song}" != "${MEDIA_PARSED_NAME}" ]; then
	print_song
	echo "${MEDIA_PARSED_NAME}"	 > "${autospam_file}"
    fi
    
    if [ "$2" == "auto" ] || [ "$3" == "auto" ]; then
	echo "/TIMER ${msg_refresh} budus macro ${play_command} ${2} ${3}"
    fi
else
    print_song
fi

#####################################################################
#____________________________________________________________________