#!/bin/bash

# buduscript_script__espeak.sh : Script per la gestione di eSpeak su
# BuDuScRiPt.

# Version 29/08/2011
# $Revision: 1.34 $
# By UDA'Software 2010

# FUNZIONE ___________________________________________________________
# Verifica l'ultimo prefisso utilizzato; se diverso, ne restituisce
# uno azzerato, altrimenti, lo salva.
# ____________________________________________________________________
function save_prefix_text {
    [ -f ${prefix_file} ] && last_prefix=$(cat ${prefix_file})
    if [ ${last_prefix:-/NULL/} == ${prefix_text:-/EMPTY/} ]; then
	prefix_text=""
    else
	printf '%s\n' "${prefix_text}" > ${prefix_file}
    fi
}

# FUNZIONE ___________________________________________________________
# Rimuove i caratteri che metterebbero in difficolta' espeak
# ____________________________________________________________________
function clean_variables() {
    channel=$(echo "${channel}" | sed 's@[^a-zA-Z0-9]@@g')
    remote_user=$(echo "${remote_user}" | sed 's@[^a-zA-Z0-9]@@g' | cut -b-8 | tr [0123456] [oizeasb] | tr [:upper:] [:lower:])
    local_user=$(echo "${local_user}" | sed 's@[^a-zA-Z0-9]@@g' | cut -b-8 | tr [0123456] [oizeasb] | tr [:upper:] [:lower:])
}

# FUNZIONE ___________________________________________________________
# Controlla se ci sono dei messaggi nella coda.
# ____________________________________________________________________
function check_queue() {
    if [ -f ${queue_file} ]; then
	is_espeak_running=$(ps -e | grep -F espeak | grep -vF -e 'buduscript_script__espeak.sh' -e '<defunct>')
	if [ "${is_espeak_running}" == "" ]; then
	    exec_command_counter=0
	    commands_queue=$(cat ${queue_file})
	    rm -f ${queue_file}
	    
	    IFS_OLD=${IFS}
	    IFS=$'\n'
	    for exec_command in ${commands_queue}; do
		if [ ${exec_command_counter} == 0 ]; then
		    local_user=$(echo "${exec_command}" | awk -F'\t' '{print $1}')
		    remote_user=$(echo "${exec_command}" | awk -F'\t' '{print $2}')
		    remote_host=$(echo "${exec_command}" | awk -F'\t' '{print $3}')
		    channel=$(echo "${exec_command}" | awk -F'\t' '{print $4}')
		    server=$(echo "${exec_command}" | awk -F'\t' '{print $5}')
		    network=$(echo "${exec_command}" | awk -F'\t' '{print $6}')
		    type=$(echo "${exec_command}" | awk -F'\t' '{print $7}')
		    data=$(echo "${exec_command}" | awk -F'\t' '{print $8}')
		    
		    flag_no_exit=1
		else
		    flag_no_exit=2
		    printf '%s\n' "${exec_command}" >> ${queue_file}
		fi
		
		exec_command_counter=$((${exec_command_counter} + 1))
	    done
	    IFS=${IFS_OLD}
	else
	    printf '/TIMER -refnum 2255 1 budus pipe_script_silent %s CHECK_QUEUE\n' "${0}"
	fi
    fi
    
    [ ${flag_no_exit:-0} == 0 ] && exit 0;
}

# FUNZIONE ___________________________________________________________
# Calcola un codice numerico da associare ad un nickname
# ____________________________________________________________________
function make_nickname_number() {
    nickname_number=0
    nickname_lower=$(echo "${remote_user}" | tr [:upper:] [:lower:])
    nickname_lenght=$(echo "${remote_user}" | wc -c)
    
    for ((char_pointer=${nickname_lenght} ; ${char_pointer} != 0 ; char_pointer= ${char_pointer} - 1)); do
	current_char=$(echo "${nickname_lower}" | cut -b${char_pointer})
	
	case "${current_char}" in
	    a)
		nickname_number=$(expr ${nickname_number} + 1);;
	    b)
		nickname_number=$(expr ${nickname_number} + 2);;
	    c)
		nickname_number=$(expr ${nickname_number} + 3);;
	    d)
		nickname_number=$(expr ${nickname_number} + 4);;
	    e)
		nickname_number=$(expr ${nickname_number} + 5);;
	    f)
		nickname_number=$(expr ${nickname_number} + 6);;
	    g)
		nickname_number=$(expr ${nickname_number} + 7);;
	    h)
		nickname_number=$(expr ${nickname_number} + 8);;
	    i)
		nickname_number=$(expr ${nickname_number} + 9);;
	    j)
		nickname_number=$(expr ${nickname_number} + 1);;
	    k)
		nickname_number=$(expr ${nickname_number} + 2);;
	    l)
		nickname_number=$(expr ${nickname_number} + 3);;
	    m)
		nickname_number=$(expr ${nickname_number} + 4);;
	    n)
		nickname_number=$(expr ${nickname_number} + 5);;
	    o)
		nickname_number=$(expr ${nickname_number} + 6);;
	    p)
		nickname_number=$(expr ${nickname_number} + 7);;
	    q)
		nickname_number=$(expr ${nickname_number} + 8);;
	    r)
		nickname_number=$(expr ${nickname_number} + 9);;
	    s)
		nickname_number=$(expr ${nickname_number} + 1);;
	    t)
		nickname_number=$(expr ${nickname_number} + 2);;
	    u)
		nickname_number=$(expr ${nickname_number} + 3);;
	    v)
		nickname_number=$(expr ${nickname_number} + 4);;
	    w)
		nickname_number=$(expr ${nickname_number} + 5);;
	    x)
		nickname_number=$(expr ${nickname_number} + 6);;
	    y)
		nickname_number=$(expr ${nickname_number} + 7);;
	    z)
		nickname_number=$(expr ${nickname_number} + 8);;
	esac
    done
}

# ____________________________________________________________________
# Funzioni per la gestione del file di configurazione
# ____________________________________________________________________
function get_var() {
    get_var=""
    
    if [ -f ${config_file} ]; then
	get_var=$(grep -F "${1}" "${config_file}" | tail -n1 | awk '{print $2}')
    fi
    
    if [ "${get_var}" == "" ]; then
	get_var="${2}"
    fi
    
    export ${1}="${get_var}"
}

# ____________________________________________________________________
function set_var() {
    printf '%s %s\n' "${1}" "${2}" >> "${config_file}"
}

# ____________________________________________________________________
function load_config() {
    get_var ESPEAK_AMPLITUDE 100
    get_var ESPEAK_WORDGAP 0
    get_var ESPEAK_VOICE it
    
    rm -f "${config_file}"

    set_var ESPEAK_AMPLITUDE "${ESPEAK_AMPLITUDE}"
    set_var ESPEAK_WORDGAP "${ESPEAK_WORDGAP}"
    set_var ESPEAK_VOICE "${ESPEAK_VOICE}"
}

# ____________________________________________________________________
# Gestione dei Parametri
# ____________________________________________________________________
local_user=$1
remote_user=$2
remote_host=$3
channel=$4
server=$5
network=$6
type=$7
data=$8

# ____________________________________________________________________
# Variabili Globali
# ____________________________________________________________________
config_file="${HOME}/.xchat2/buduscript/espeak.config"
user_home=$(echo "${HOME}" | tr [:punct:] '_')
queue_file="/tmp/buduscript_script__espeak_queue${user_home}.dat"
prefix_file="/tmp/buduscript_script__espeak_prefix${user_home}.dat"

# ____________________________________________________________________
# Verifica se e' stato invocato solo per creare il menu delle voci
# disponibili.
# ____________________________________________________________________
if [ "${local_user}" == "CHECK_QUEUE" ]; then
    check_queue
elif [ "${local_user}" == "DELETE_QUEUE" ]; then
    printf '/ECHO eSpeak Queue Erased !\n'
    rm -f ${queue_file}
    
    exit 0;
elif [ "${local_user}" == "MAKE_AVAILABLE_VOICE_RADIO_MENU" ]; then
    [ -f ${queue_file} ] && rm -f ${queue_file}
    
    if [ $(which espeak 2> /dev/null) ]; then
	load_config
	
	available_language=$(espeak --voices | awk '{print $2"\t"$4}' | grep -vF 'Language' | sort)
	xchat_espeak_menu=""
	
	IFS_OLD=${IFS}
	IFS=$'\n'
	espeak_menu=$(
	for language in ${available_language}; do
	    language_ID=$(echo "${language}" | awk -F'\t' '{print $1}')
	    language_value=$(echo "${language}" | awk -F'\t' '{print $2}' | sed 's@[^a-zA-Z0-9 ]@ @g')
	    if test ${language_ID} == ${ESPEAK_VOICE} ; then
		language_flag=1
		radio_button_name="${language_ID} - ${language_value}"
	    else
		language_flag=0
		radio_button_name="_BUDUSCRIPT_ESPEAK_RADIO_BUTTON_NAME_"
	    fi
	    
	    printf '/MENU -r%d,"%s" ADD "eSpeak/Configurazione/Lingua/%s - %s" "budus pipe_script_silent echo '\''ESPEAK_VOICE %s'\'' >> ${HOME}/.xchat2/buduscript/espeak.config"\n' "${language_flag}" "${radio_button_name}" "${language_ID}" "${language_value}" "${language_ID}"
	done
	)

	echo "${espeak_menu}" | grep -F -- ' -r1,'
	radio_button_name=$(echo "${espeak_menu}" | grep -F -- ' -r1,' | awk -F'"' '{print $2}')
	printf '/MENU ADD "eSpeak/Configurazione/Lingua/-\n'
	echo "${espeak_menu}" | grep -F -- ' -r0,' | sed "s@_BUDUSCRIPT_ESPEAK_RADIO_BUTTON_NAME_@${radio_button_name}@g"
	
	IFS=${IFS_OLD}
    else
	echo "/ECHO %C4,1%BERRORE:%C8,1 eSpeak non rilevato !"
    fi
    exit 0;
fi

# ____________________________________________________________________
# Verifica che l'esecuzione dello script non sia troppo ravvicinata.
# ____________________________________________________________________
is_espeak_running=$(ps -e | grep espeak | grep -vF -e 'buduscript_script__espeak.sh' -e '<defunct>')
if [ "${is_espeak_running}" != "" ]; then
    [ -f ${queue_file} ] && buduscript_script__espeak_queue=$(cat ${queue_file} | wc -l)
    
    if [ ${buduscript_script__espeak_queue:-0} -gt 8 ]; then
	echo "eSpeak running [ ${buduscript_script__espeak_queue:-0} ] !" >&2
	exit 1;
    else
	printf '%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n' "${local_user}" "${remote_user}" "${remote_host}" "${channel}" "${server}" "${network}" "${type}" "${data}" >> ${queue_file}
	
	printf '/TIMER -refnum 2255 1 budus pipe_script_silent %s CHECK_QUEUE\n' "${0}"
	
	exit 0;
    fi
elif [ ${flag_no_exit:-0} == 0 ]; then
    flag_no_exit=1
    check_queue
    if [ ${flag_no_exit} == 2 ]; then
	printf '/TIMER -refnum 2255 1 budus pipe_script_silent %s CHECK_QUEUE\n' "${0}"
    fi
fi

# ____________________________________________________________________
# Carica la configurazione.
# ____________________________________________________________________
load_config

# ____________________________________________________________________
# Analizza la frase da "parlare".
# ____________________________________________________________________

# Converte gli slash
data=$(echo "${data}" | sed 's@\\@slsh@g')

# Elimina i link
data=$(echo "${data}" | sed 's@[a-zA-Z0-9]*://.*[ ]@ link @g')

# Elimina i simboli doppi
data=$(echo "${data}" | tr -s '[\|!"$%&/()=?^+*#@\[\]{}.:,;-_<>`]' '[\|!"$%&/()=?^+*#@\[\]{}.:,;-_<>`]')

# Elimina i caratteri accentati doppi
data=$(echo "${data}" | tr -s '[]' '[]')

# Riduce i caratteri ripetuti piu' di due volte.
data_lenght=$(echo "${data}" | wc -c)
for ((char_pointer=1 ; ${char_pointer} != ${data_lenght} ; char_pointer= ${char_pointer} + 1)); do
    current_char=$(echo "${data}" | cut -b${char_pointer})
    if [ "${current_char}" == "${previous_char}" ]; then
	match_repetition=$((${match_repetition:-0} + 1))
    else
	previous_char="${current_char}"
	match_repetition=0
    fi
    
    if [ ${match_repetition} -le 1 ]; then
	new_data="${new_data}${current_char}"
    fi
done

# Elimina gli url
data=$(echo "${data}" | sed 's@http:/[^ ]*@ url @g')

# Elimina le "parole" oltre i 16 caratteri.
data=$(echo "${new_data}" | sed 's@[^ ]\{16,\}@_cut_@')

# Converte alcune parole classiche italiane
data=$(echo "${data}" | sed 's@e[tc]c[\.]\+@eccetera@g')

# Elimina i numeri troppo lunghi
data=$(echo "${data}" | sed 's@[0-9]\{16,\}@@g')

# Tronca le frasi piu' lunghe di 256 caratteri
data=$(echo "${data}" | cut -b-255)

# Escape delle virgolette.
data=$(echo "${data}" | sed 's@"@virgolette@g')

# Escape dei "dollari"
data=$(echo "${data}" | sed 's@\$@dollaro@g')

# Escape degli apici inversi
data=$(echo "${data}" | sed 's@`@@g')

# Converte gli smiles classici
data=$(echo "${data}" | sed 's@:)@ sorrde @g')
data=$(echo "${data}" | sed 's@:\*@ bacio @g')
data=$(echo "${data}" | sed 's@=)@ sorrde @g')
data=$(echo "${data}" | sed 's@:(@ triste @g')
data=$(echo "${data}" | sed 's@:D@ ride @g')
data=$(echo "${data}" | sed 's@XD@ ride troppo @g')
data=$(echo "${data}" | sed 's@:S@ disgustato @g')
data=$(echo "${data}" | sed 's@:/@ perplesso @g')
data=$(echo "${data}" | sed 's@:O@ sbalordito @g')
data=$(echo "${data}" | sed 's@:X@ ammutolito @g')
data=$(echo "${data}" | sed 's@>:@ arrabbiato @g')
data=$(echo "${data}" | sed 's@:>@ ghigno @g')
data=$(echo "${data}" | sed 's@:E@ vampiro @g')
data=$(echo "${data}" | sed 's@:P@ linguaccia @g')
data=$(echo "${data}" | sed 's@:p@ lingua @g')
data=$(echo "${data}" | sed 's@:@ piange @g')
data=$(echo "${data}" | sed 's@:\.@ piange @g')
data=$(echo "${data}" | sed 's@:|@ serio @g')
data=$(echo "${data}" | sed 's@:_@ piange @g')
data=$(echo "${data}" | sed 's@;)@ occhiolino @g')
data=$(echo "${data}" | sed 's@-\.-@ tapinizzto @g')

# Converte alcune parole classiche inglesi
data=$(echo "${data}" | sed 's@ok@chi@g')

# ____________________________________________________________________
# Verifica il tipo di invocazione dello script
# ____________________________________________________________________
case ${type} in
    jchannel)
	clean_variables
	text="${remote_user}, entrato in, ${channel} ."
	;;
    kick)
	clean_variables
	text="${local_user}, cacciato da, ${channel} ."
	;;
    pserver)
	clean_variables
	text="${remote_user}, disconnesso."
	;;
    pchannel)
	clean_variables
	text="${remote_user}, uscito da, ${channel} ."
	;;
    chgnick)
	clean_variables
	text="In ${channel}, ${local_user} ha cambiato nick in ${remote_user} ."
	;;
    mode)
	clean_variables
	case ${data} in
	    +o)
		text="In ${channel}, ${local_user} promosso operatore."
		;;
	    -o)
		text="In ${channel}, ${local_user} rimosso da operatore."
		;;
	    +h)
		text="In ${channel}, ${local_user} promosso mezzo operatore."
		;;
	    -h)
		text="In ${channel}, ${local_user} rimosso da mezzo operatore."
		;;
	    +v)
		text="In ${channel}, ${local_user} pu parlare."
		;;
	    -v)
		text="In ${channel}, ${local_user}  meglio se sta zitto."
		;;
	    +b)
		text="In ${channel}, ${remote_user} ha messo un BAN."
		;;
	    -b)
		text="In ${channel}, ${remote_user} ha tolto un BAN."
		;;
	    *)
		text="In ${channel}, applicata modalit ${data} a ${remote_user} ."
		;;
	esac
	;;
    flood)
	clean_variables
	text="In ${channel}, ${remote_user} sta scrivendo troppo."
	;;
    badword)
	clean_variables
	text="In ${channel}, ${remote_user} sta scrivendo parolacce."
	;;
    badnick)
	clean_variables
	text="In ${channel}, ${remote_user} usa un nickname proibito."
	;;
    notify)
	clean_variables
	case ${data} in
	    "logged online")
		text="${remote_user} nlin sul server ${server}."
		;;
	    "is online")
		text="${remote_user} nlin sul server ${server}."
		;;
	    "logged offline")
		text="${remote_user} scollegato dal server ${server}."
		;;
	    "is offline")
		text="${remote_user} scollegato dal server ${server}."
		;;
	    *)
		text="${remote_user} ha generato una notifica sul server ${server}."
		;;
	esac
	;;
    invite)
	clean_variables
	data=$(echo "${data}" | sed 's@[^a-zA-Z0-9]@@g')
	text="${remote_user} ti ha invitato in ${data}."
	;;
    text)
	clean_variables
	prefix_text="${channel}. . ${remote_user}. . "
	save_prefix_text
	text="${prefix_text}${data}"
	;;
    text_user)
	clean_variables
	prefix_text="${channel}. ${remote_user}, "
	save_prefix_text
	text="${prefix_text}${data}"
	;;
    text_user_private)
	clean_variables
	save_prefix_text
	text="QUERI. . ${data}"
	;;
    text_user_channel)
	clean_variables
	prefix_text="${remote_user}, "
	save_prefix_text
	text="${prefix_text}${data}"
	;;
    text_channel)
	clean_variables
	prefix_text="${remote_user}, "
	save_prefix_text
	text="${prefix_text}${data}"
	;;
    *)
	printf '/ECHO eSpeak DEBUG INFO: local_user=%s, remote_user=%s, remote_host=%s, channel=%s, server=%s, network=%s, type=%s, data=%s\n' "${local_user}" "${remote_user}" "${remote_host}" "${channel}" "${server}" "${network}" "${type}" "${data}"
	text="Errore interno dello script."
	;;
esac


# ____________________________________________________________________
# Esegue eSpeak
# ____________________________________________________________________
echo "/budus exec espeak -a ${ESPEAK_AMPLITUDE} -g ${ESPEAK_WORDGAP} -v ${ESPEAK_VOICE} -- \"${text}\" > /dev/null 2> /dev/null"
